import org.apache.tinkerpop.gremlin.process.traversal.dsl.graph.GraphTraversal
import org.apache.tinkerpop.gremlin.structure.*;

def getOrCreateVertex(arg, graph, createdVertices, createIfNotExist, anonVertexLabels) {
    if (arg instanceof Map) {
        Vertex v = graph.vertex(arg)

        if(arg.containsKey("community_id") && arg.containsKey("member_id")) {
            return v
        }
        else {
            if( v == null ) {
                return graph.addVertex(arg);
            }
            return v
        }

    }

    //else it's an anonymous vertex in this batch
    int position = ((Number)arg).intValue();
    Vertex vertex = createdVertices[position];
    if (vertex == null && createIfNotExist) {
        //Create the vertex on demand
        vertex = graph.addVertex(T.label, anonVertexLabels[position]);
        createdVertices[position]=vertex;
    }
    return vertex;
}

def setProperties(Element element, Map properties) {
    for (Map.Entry prop : properties.entrySet()) {
        element.property(prop.getKey(), prop.getValue());
    }
}

def qualifyTraversal(GraphTraversal<Vertex,Edge> traversal, keys, Map properties) {
    if (!keys.isEmpty()) {
        for (String key : keys) {
            traversal = traversal.has(key, properties.get(key));
        }
    } else { //Use all properties
        for (Map.Entry prop : properties.entrySet()) {
            traversal = traversal.has(prop.getKey(), prop.getValue());
        }
    }
    return traversal;
}

Vertex[] anonVertices = new Vertex[anonymous.size()];
int num = 0;

for (List edge: edges) {
    boolean isAbsent = edge[5];
    String label = edge[1];
    if (graph.schemaModel().edgeLabel(label).cardinality().vertexPropertyCardinality()==VertexProperty.Cardinality.single) {
        isAbsent = true;
    }
    Edge addedEdge = null;
    if (!isAbsent) { //Try to find edge if it exists
        def vout = getOrCreateVertex(edge[0], graph, anonVertices, false, anonymous);
        def vin = getOrCreateVertex(edge[2], graph, anonVertices, false, anonymous);

        GraphTraversal traversal;
        if (vout!=null) {
            traversal = qualifyTraversal(g.V(vout).outE(label), edge[4], edge[3]);
            if (vin!=null) traversal = traversal.as("e").inV().is(vin).select("e");


        } else if (vin!=null) {
            traversal = qualifyTraversal(g.V(vin).inE(label), edge[4], edge[3]);
        } else {
            //If both are null, then these are two anonymous vertices that haven't been instantiated and hence this edge should be ignored.
            continue;
        }
        addedEdge = traversal.tryNext().orElse(null);
    }
    if (addedEdge == null) {
        def vout = getOrCreateVertex(edge[0], graph, anonVertices, true, anonymous);
        def vin = getOrCreateVertex(edge[2], graph, anonVertices, true, anonymous);
        addedEdge = vout.addEdge(label, vin);
        num++;
    }
    setProperties(addedEdge, edge[3]);
}


for (List vprop: properties) {
    def v = getOrCreateVertex(vprop[0], graph, anonVertices, false, anonymous);
    if (v != null) {
        boolean isAbsent = vprop[5];
        String key = vprop[1];
        Object value = vprop[2];
        if (graph.schemaModel().propertyKey(key).cardinality().vertexPropertyCardinality()==VertexProperty.Cardinality.single) {
            isAbsent = true;
        }
        VertexProperty addedProperty = null;
        if (!isAbsent) { //Try to find the vertex property if it exists
            GraphTraversal traversal = qualifyTraversal(g.V(v).properties(key), vprop[4], vprop[3]);
            addedProperty = traversal.hasValue(value).tryNext().orElse(null);
        }

        if (addedProperty == null) {
            addedProperty = v.property(key, value);
            num++;
        }
        setProperties(addedProperty, vprop[3]);
    }
}


return num;
